﻿using GHML;
using HarmonyLib;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class PlayerCompass : Mod
{
    RawImage CompassImage;
    Text CompassDirectionText;
    float compassUnit = 1.06f;
    GameObject PlayerEntry;
    GameObject canvas;
    GameObject playerparent;
    AssetBundle asset;
    public static PlayerCompass instance;

    GameObject wpparent;
    GameObject markerMenu;
    InputField markerName;
    GameObject markerlistContent;
    GameObject markerlistEntry;
    GameObject markerBeam;
    GameObject waypointText;

    Harmony harmonyInstance;

    public List<Waypoint> waypoints = new List<Waypoint>();

    public class MarkerBeam : MonoBehaviour
    {

    }

    public class Waypoint
    {
        public string name;
        public float x;
        public float y;
        public float z;
        public bool showBeam;

        public Waypoint(string n, Vector3 pos, bool beam)
        {
            name = n;
            x = pos.x;
            y = pos.y;
            z = pos.z;
            showBeam = beam;
        }
    }

    public void Start()
    {
        harmonyInstance = new Harmony("fr.tekgamer.playercompass");
        harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
        instance = this;
        asset = AssetBundle.LoadFromMemory(GetEmbeddedFileBytes("playercompass.assets"));
        canvas = Instantiate(asset.LoadAsset<GameObject>("CompassCanvas"), transform);
        markerMenu = Instantiate(asset.LoadAsset<GameObject>("WaypointManager"), transform);
        markerlistEntry = asset.LoadAsset<GameObject>("WaypointMenuEntry");
        markerBeam = asset.LoadAsset<GameObject>("MarkerBeam");
        markerMenu.SetActive(false);
        markerName = markerMenu.transform.Find("BG").Find("MarkerNameInputfield").GetComponent<InputField>();
        markerlistContent = markerMenu.transform.Find("BG").Find("Scroll View").Find("Viewport").Find("Content").gameObject;
        markerMenu.transform.Find("BG").Find("CloseBtn").GetComponent<Button>().onClick.AddListener(() =>
        {
            CloseMenu();
        });
        markerMenu.transform.Find("BG").Find("AddMarkerBtn").GetComponent<Button>().onClick.AddListener(() =>
        {
            AddMarker();
        });
        playerparent = canvas.transform.Find("PlayerParent").gameObject;
        wpparent = canvas.transform.Find("WPParent").gameObject;
        CompassImage = canvas.transform.Find("Background").Find("CompassTexture").GetComponent<RawImage>();
        CompassDirectionText = canvas.transform.Find("MiddleText").GetComponent<Text>();
        PlayerEntry = asset.LoadAsset<GameObject>("PlayerEntry");
        waypointText = asset.LoadAsset<GameObject>("WaypointEntry");
        if (GHAPI.IsCurrentSceneGame())
        {
            LoadWaypoints();
        }
        Debug.Log("Mod PlayerCompass has been loaded!");
    }

    public void AddMarker()
    {
        if (!GHAPI.IsCurrentSceneGame()) return;
        if (markerName.text.Length > 2 && waypoints.Where(x => x.name == markerName.text).Count() == 0)
        {
            waypoints.Add(new Waypoint(markerName.text, GHAPI.GetLocalPlayer().transform.position, true));
            UpdateWaypoints();
            SaveWaypoints();
        }
    }

    public void UpdateWaypoints()
    {
        foreach (Transform t in markerlistContent.transform)
        {
            Destroy(t.gameObject);
        }

        GameObject.FindObjectsOfType<MarkerBeam>().ToList().ForEach(x =>
        {
            Destroy(x.gameObject);
        });

        foreach (Waypoint w in waypoints)
        {
            GameObject entry = Instantiate(markerlistEntry, markerlistContent.transform);
            entry.transform.Find("Name").GetComponent<Text>().text = w.name;
            entry.transform.Find("ShowBeamToggle").GetComponent<Toggle>().isOn = w.showBeam;
            entry.transform.Find("ShowBeamToggle").GetComponent<Toggle>().onValueChanged.AddListener((x) =>
            {
                waypoints.Find(a => a.name == w.name).showBeam = x;
                UpdateWaypoints();
            });
            entry.transform.Find("DeleteBtn").GetComponent<Button>().onClick.AddListener(() =>
            {
                DeleteMarker(w.name);
            });
            if (w.showBeam)
            {
                GameObject beam = Instantiate(markerBeam, new Vector3(w.x, w.y, w.z), Quaternion.identity);
                beam.AddComponent<MarkerBeam>();
            }
        }

    }

    public void DeleteMarker(string name)
    {
        waypoints = waypoints.Where(x => x.name != name).ToList();
        UpdateWaypoints();
        SaveWaypoints();
    }

    public void SaveWaypoints()
    {
        string data = JsonConvert.SerializeObject(waypoints);
        string name = SaveGame.s_MainSaveName;
        PlayerPrefs.SetString("CompassMarkers_" + name, data);
    }

    public void LoadWaypoints()
    {
        waypoints = new List<Waypoint>();
        string name = SaveGame.s_MainSaveName;
        if (PlayerPrefs.HasKey("CompassMarkers_" + name))
        {
            try
            {
                string data = PlayerPrefs.GetString("CompassMarkers_" + name);
                List<Waypoint> wp = JsonConvert.DeserializeObject<List<Waypoint>>(data);
                waypoints = wp;
                UpdateWaypoints();
            }
            catch (Exception e)
            {
                Debug.LogError("Could not load saved compass markers !\n" + e); waypoints = new List<Waypoint>();
                SaveWaypoints();
                UpdateWaypoints();
            }
        }
    }

    public void CloseMenu()
    {
        markerMenu.SetActive(false);
        InternalGHAPI.GHInternal_ShowCursor(false);
    }

    [ConsoleCommand("markermenu", "Opens the marker menu that allows you to add and edit markers/waypoints to the compass.")]
    public static void markermenucommand()
    {
        if (!GHAPI.IsCurrentSceneGame()) return;
        instance.OpenMenu();
    }

    public void OpenMenu()
    {
        UpdateWaypoints();
        if (!markerMenu.activeSelf)
        {
            markerMenu.SetActive(true);
            InternalGHAPI.GHInternal_ShowCursor(true);
        }
        else
        {
            CloseMenu();
        }
    }

    public void Update()
    {
        if (SceneManager.GetActiveScene().name != "Level" || Player.Get() == null) { canvas.SetActive(false); return; }
        canvas.SetActive(true);
        List<P2PPeer> connectedPlayers = ReplTools.GetRemotePeers().ToList();

        connectedPlayers.ForEach(player =>
        {
            if (player.m_Address != null && player.m_Representation != null)
            {
                Transform text = playerparent.transform.Find("Player_" + player.m_Address);
                if (text == null)
                {
                    text = Instantiate(PlayerEntry, playerparent.transform).transform;
                    text.name = "Player_" + player.m_Address;
                }

                text.GetComponent<RectTransform>().anchoredPosition = GetPosOnCompass(player.m_Representation.GetWorldPosition());
                text.GetComponent<Text>().text = player.GetDisplayName() + "\n<b>" + Mathf.RoundToInt(Vector3.Distance(Player.Get().transform.position, player.m_Representation.GetWorldPosition())) + "m</b>";
            }
        });

        waypoints.ForEach(wp =>
        {
            Transform text = wpparent.transform.Find("WP_" + wp.name);
            if (text == null)
            {
                text = Instantiate(waypointText, wpparent.transform).transform;
                text.name = "WP_" + wp.name;
            }

            Vector3 pos = new Vector3(wp.x, wp.y, wp.z);
            text.GetComponent<RectTransform>().anchoredPosition = GetPosOnCompass(pos);
            text.GetComponent<Text>().text = wp.name + "\n<b>" + Mathf.RoundToInt(Vector3.Distance(Player.Get().transform.position, pos)) + "m</b>";
        });

        foreach (Transform child in wpparent.transform)
        {
            if (!waypoints.Where(p => "WP_" + p.name == child.name).Any())
            {
                Destroy(child.gameObject);
            }
        }

        foreach (Transform child in playerparent.transform)
        {
            if (!connectedPlayers.Where(p => "Player_" + p.m_Address == child.name).Any())
            {
                Destroy(child.gameObject);
            }
        }

        CompassImage.uvRect = new Rect(Player.Get().transform.localEulerAngles.y / 360, 0, 1, 1);
        Vector3 forward = Player.Get().transform.forward;
        forward.y = 0;
        float headingAngle = Quaternion.LookRotation(forward).eulerAngles.y;
        headingAngle = 5 * (Mathf.RoundToInt(headingAngle / 5.0f));
        int displayangle;
        displayangle = Mathf.RoundToInt(headingAngle);
        switch (displayangle)
        {
            case 0:
                CompassDirectionText.text = "N";
                break;
            case 360:
                CompassDirectionText.text = "N";
                break;
            case 45:
                //Do this
                CompassDirectionText.text = "NE";
                break;
            case 90:
                CompassDirectionText.text = "E";
                break;
            case 130:
                CompassDirectionText.text = "SE";
                break;
            case 180:
                CompassDirectionText.text = "S";
                break;
            case 225:
                CompassDirectionText.text = "SW";
                break;
            case 270:
                CompassDirectionText.text = "W";
                break;
            default:
                CompassDirectionText.text = headingAngle.ToString();
                break;
        }
    }

    Vector2 GetPosOnCompass(Vector3 pos)
    {
        Vector2 playerPos = new Vector2(Player.Get().transform.position.x, Player.Get().transform.position.z);
        Vector2 playerFwd = new Vector2(Player.Get().transform.forward.x, Player.Get().transform.forward.z);
        float angle = Vector2.SignedAngle(new Vector2(pos.x, pos.z) - playerPos, playerFwd);
        return new Vector2(compassUnit * angle, -105f);
    }

    public void OnModUnload()
    {
        harmonyInstance.UnpatchAll("fr.tekgamer.playercompass");
        if (asset != null)
            asset.Unload(true);
        GameObject.FindObjectsOfType<MarkerBeam>().ToList().ForEach(x =>
        {
            Destroy(x.gameObject);
        });
        Debug.Log("Mod PlayerCompass has been unloaded!");
    }
}

[HarmonyPatch(typeof(SaveGame))]
[HarmonyPatch("Load", new Type[] { })]
internal class Patch_OnLoadSave
{
    public async static void Postfix()
    {
        await Task.Delay(3000);
        string savename = SaveGame.s_MainSaveName;
        Debug.Log("[PlayerCompass] Loading saved waypoints...");
        PlayerCompass.instance.LoadWaypoints();
        Debug.Log("[PlayerCompass] Loaded saved waypoints.");
    }
}